/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* pa_base.c (Enumeration of all partitions of a set)
   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:31  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1998/04/15  18:36:04  marzetta
 * Initial revision
 *
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "pa_base.h"

/* Exported variables -----------------------------------*/

int pa_NodeTag, pa_GlobalTag, pa_TotalTag;


/* Static variables -------------------------------------*/

static boolean pa_inited;


/* Procedures -------------------------------------------*/

pa_NodeP pa_NewNode(int n)
{
  pa_NodeP np;

  np = (pa_NodeP) z_NewNode(pa_NodeTag,
			    sizeof(pa_Node) + n * sizeof(pa_dataT));
  return np;
} /* pa_NewNode */

void pa_FreeNode(pa_NodeP node)
{
  z_FreeNode((z_NodeP) node);
} /* pa_FreeNode */

void pa_Print(pa_GlobalP global, pa_NodeP node)
{
  int i, class;
  boolean firstofclass, firstofline;
  
  if (node->unassigned) {
    printf("a = %ld, b = %ld, act_num = %ld, act_elt = %ld:",
	   (long)node->a, (long)node->b,
	   (long)node->act_num, (long)node->act_elt);
    for (i=0; i<global->n; i++) printf("%2d", pa_class(node, i));
  } else {
    firstofline = 1;
    for (class=1; class<=global->n; class++) {
      firstofclass = 1;
      for (i=0; i<global->n; i++)
	if (pa_class(node, i) == class) {
	  printf("%s%d", (firstofclass ? (firstofline ? "{" : "; {") : ", "),
		 i+1);
	  firstofclass = firstofline = 0;
	}
      if (!firstofclass) printf("}");
    }
  }
  printf("\n");
} /* pa_Print */

void pa_base_init(void)
{
  /* Initialize this module */
  if (! pa_inited) {
    pa_inited = 1;
    pa_NodeTag = z_InstallClass(pa_Node, 0, 0, 0);
    pa_GlobalTag = z_InstallClass(pa_Global, 0, 0, 0);
    pa_TotalTag = z_InstallClass(pa_Total, 0, 0, 0);
  }
  
}
