/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* pa_base.h (Enumeration of all partitions of a set)
   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:31  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1998/04/15  18:36:04  marzetta
 * Initial revision
 *
 */

#include "z_base.h"

#define MaxN 35

typedef uint16 pa_dataT;

extern int pa_NodeTag, pa_GlobalTag, pa_TotalTag;

typedef struct pa_Node { /* opaque, a node of the tree to be searched */
  Z_NODE_HEADER;                /* len; tag = pa_NodeTag */
  uint32 a, b;			/* enumerate partitions with a..b classes */
  uint16 act_num;		/* number of actual class */
  uint16 act_elt;		/* element for actual class */
  uint16 unassigned;		/* redundant (# entries in pa_class == 0) */
  pa_dataT data[1];
} pa_Node, *pa_NodeP;

#define pa_class(n, i)	(n->data[i])

/* Elements < act_elt have been considered for the class act_num.
   If act_elt == 0, the actual class is empty.
   Enumerate partitions with a..b classes including class act_num.
 */
typedef struct pa_Global {
  Z_NODE_HEADER;                /* len; tag = pa_GlobalTag */
  uint32 n;			/* must be > 0 */
  boolean quiet;
} pa_Global, *pa_GlobalP;

typedef struct pa_Total {
  Z_NODE_HEADER;
  long sol_count;		/* count of solutions */
} pa_Total, *pa_TotalP;

/* Procedures ------------------------------------------------------- */

pa_NodeP pa_NewNode(int n);

void pa_FreeNode(pa_NodeP node);

void pa_Print(pa_GlobalP global, pa_NodeP node);

void pa_base_init(void);
