/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* z_lb.h (ZRAM: General load balancing)
   Ambros Marzetta
   95-09-20

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.4  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.3  2001/08/17 13:24:42  bremner
 * checkpointing not obviously broken
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.3  1998/04/15  18:34:56  marzetta
 * Added z_lb_redundantComputation.
 *
 * Revision 1.2  1996/05/29  11:07:36  marzetta
 * New statistics
 *
 * Revision 1.1  1996/05/08  20:29:30  marzetta
 * Initial revision
 *
 */

#ifndef _Z_LB_H_
#define _Z_LB_H_
#include <limits.h>
#include "z_base.h"
#include "z_ps.h"
#include "z_par.h"		/* z_VoidProc */
#include "s_seq.h"
#include "z_io.h"

/* Variables ------------------------------------------------------------ */

extern char z_lb_CheckpointPath[PATH_MAX];
     				/* name of checkpoint file ("check") */
extern boolean z_lb_traceCheckpoint;
				/* trace checkpoints */
extern boolean z_lb_writeCheckpoint;
				/* write checkpoint files */
extern boolean z_lb_writeStatistics;
				/* write statistics to stdout */
extern boolean z_lb_backupCheckpoint;
				/* Make a backup of the checkpoint file. */
				/* This is more safe, but needs twice */
				/* as much disk space. */

extern boolean z_lb_redundantComputation; /* read-only for application */
/* This variable is set to true during the startup phase of the load-
   balancing algorithm when several processors evaluate WorkAndSplit
   for the same argument. */

extern boolean z_lb_needSplit;	/* read-only for application */
extern boolean z_lb_needPoll;	/* read-only for application */

/* Types ---------------------------------------------------------------- */

#ifdef _Z_LB_C_
typedef z_NodeP z_lb_GlobalP, z_lb_WorkUnitP;
#else
typedef void *z_lb_GlobalP, *z_lb_WorkUnitP; /* z_NodeP */
#endif

typedef void (*z_lb_WorkProc)(z_lb_GlobalP global, z_lb_WorkUnitP *workUnitH);
/* Deallocate *nodeH.
   Allocate its successor or
   set *nodeH to NULL if it was the last node. */

typedef void (*z_lb_SplitProc)(z_lb_GlobalP global,
			       z_lb_WorkUnitP workUnit,
			       s_seqP workList);
/* Deallocate workUnit. Allocate parts of it and insert them into workList.
 */

/* lb_WorkAndSplit(global, workUnit, workList)
   Deallocate workUnit. Allocate workUnits and insert them into workList.
   */

typedef struct z_lb_ProcSet {
  z_lb_WorkProc		lb_Work;
  z_lb_SplitProc	lb_Split;
  z_lb_SplitProc	lb_WorkAndSplit;
  z_VoidProc		lb_Fini; /* may be NULL */
} z_lb_ProcSet, *z_lb_ProcSetP;

/* The work units of every processor are kept in a sequence. The
   algorithm always works on the last element of the sequence. When a
   processor is out of work, it gets the first element of another
   processor's work list. For a depth-first search WorkAndSplit must
   insert a node's children at the end of the sequence. */

/* Procedures ----------------------------------------------------------- */

void z_lb_Work(z_ProcSetNum procsetnum,
	       z_lb_GlobalP global,
	       z_lb_WorkUnitP workUnit,
	       boolean sequential);

void z_lb_RestartWork(z_ProcSetNum procsetnum,
		      z_lb_GlobalP global,
		      boolean sequential);

void z_lb_InstallCheckpointed(z_NodeP *nodeh);

/* Initialization ------------------------------------------------------- */

void z_lb_Init(void);
/* Initialize this module */

void z_lb_setCheckpointName(char *);
#endif /* _Z_LB_H_ */
