/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* ZRAM: Message passing with termination detection
   To be used for communication between PEs, not with the FE.

   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.4  1998/03/10  15:02:16  marzetta
 * z_Poll() is now a boolean function rather than void.
 *
 * Revision 1.3  1996/04/09  13:24:10  marzetta
 * Added new Install procedures and SEQUENTIAL
 *
 * Revision 1.2  1995/08/22  12:39:45  marzetta
 * New message passing, MPI
 *
 * Revision 1.1  1995/07/27  11:38:34  marzetta
 * Initial revision
 *
 */

#ifndef SEQUENTIAL
#ifndef _Z_MP_H_
#define _Z_MP_H_
#include "z_par.h"

#define z_InstallMessage(handler) z__InstallMessage(handler, #handler)

z_tagT z__InstallMessage(z_VoidProc handler, const char *name);
/* Install a handler for incoming messages with some tag.
   Collective operation. Should be called from initproc().
   
   An installed message handler must not call z_IdleWithHandler() or
   z_Poll(), i.e. while a message handler is running, no messages
   can be received.
   */

void z_StartMessagePassing(void);
/* Start message passing and the termination detection algorithm.
   */

boolean z_TestSend(void);
/* Is it possible to overwrite outbuffer?
   Pre: z_StartMessagePassing has been called.
   */

void z_Isend(int dest, char *comment);
/* Send outbuffer to dest.
   Pre: dest != myproc, z_TestSend()
   */

void z_Ibroadcast(char *comment);
/* Send outbuffer to all nodes except myproc.
   Pre: z_TestSend()
   */

void z_QueueSender(z_VoidProc sender);
/* Insert sender into the send queue. As soon as sending becomes possible,
   the first sender from the queue will be called. Abort the program if
   the send queue overflows.
   The sender may call z_Isend, z_Ibroadcast and z_QueueSender.
   */

boolean z_IdleWithHandler(boolean sureReceive);
/* The processing element is idle.
   If a sender is queued and sending becomes possible,
            call the sender and return false.
   If a message is received, call its handler and return false.
   If all PEs are idle and (! sureReceive), return true.

   Setting sureReceive to false is always correct. By setting it to true
   one can gain some efficiency by temporarily deactivating the termination
   detection algorithm.
   */

boolean z_Poll(void);
/* If a message is received, call its handler.
   If sending is possible and a sender is queued, call it.
   Return 1 if either of those happened.
   */

/* Initialization --------------------------------------------------- */

void z_mp_Init(void);
/* Must be called from initproc(). */

#endif /* _Z_MP_H_ */
#endif /* SEQUENTIAL */
